package papertoolkit.pen.synch;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.net.ConnectException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import javax.swing.JOptionPane;
import javax.swing.JPasswordField;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.UserInfo;


import papertoolkit.PaperToolkit;
import papertoolkit.util.DebugUtils;
import papertoolkit.util.SystemUtils;
import papertoolkit.util.classpath.EclipseProjectClassLoader;
import papertoolkit.util.files.FileUtils;

/**
 * <p>
 * The batched importer will be called by the pen importer (.NET code) every time you synchronize the pen. It
 * sends the information to the BatchServer, which will notify any running applications.
 * 
 * TODO: In the future, applications do not need to be running all the time. They will be notified of new data
 * upon booting.
 * 
 * This class is launched by PaperToolkit\penSynch\bin\BatchImporter.exe. It creates a log file in the same
 * directory, that you can tail using an app such as BareTail.
 * </p>
 * <p>
 * <span class="BSDLicense"> This software is distributed under the <a
 * href="http://hci.stanford.edu/research/copyright.txt">BSD License</a>. </span>
 * </p>
 * 
 * @author <a href="http://graphics.stanford.edu/~ronyeh">Ron B Yeh</a> (ronyeh(AT)cs.stanford.edu)
 */
public class BatchedDataImporter {

	/**
	 * Pops up a Dialog to tell us this Importer was run...
	 */
	private static final boolean DEBUG = false;

	/**
	 * @param args
	 */
	public BatchedDataImporter(String[] args) {
		try {
			// redirect system output to a log file
			System.setOut(new PrintStream(new FileOutputStream(new File(PaperToolkit.getToolkitRootPath(),
					"penSynch/bin/BatchImporter.log"))));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		
		DebugUtils.println("Current Time is: " + new Date());

		if (DEBUG) {
			JOptionPane.showMessageDialog(null, "Batched Pen Data saved to penSynch/Data/XML/."
					+ "\nClick OK to Notify Registered Listeners...");
		}

		// The simpler way figures out which app to run...
		sendToRegisteredMonitors(args);

		if (DEBUG) {
			JOptionPane.showMessageDialog(null, "Batched Pen Data saved to penSynch/Data/XML/."
					+ "\nClick OK to Notify Live Listeners...");
		}

		// This alternative way talks to an already-running application. We should preserve it...
		sendToBatchedDataServer(args);
		
		saveInWebServer(args);
		
		saveInSharedFolder(args, "penSynch/data-config.txt");
		
		System.exit(0);
	}

	private void saveInSharedFolder(String args[], String configFilename) {			
		String destinationFolder = null;
		// read WWW-server connection data from configuration file
		try {
			BufferedReader configFile = new BufferedReader(new FileReader(new File(PaperToolkit.getToolkitRootPath(),
					configFilename)));
			
			destinationFolder = configFile.readLine() + "/";
			configFile.close();
			
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace(); 	
			return;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace(); 
			return;
		}
		
		/////////
		for(int i=0; i<args.length; ++i){
			String rfile = destinationFolder + (new File(args[i]).getName());
			
			OutputStream out;
			InputStream in;
			try {
				out = new FileOutputStream(rfile);
				in = new FileInputStream(args[i]);
				
				byte[] buf=new byte[1024];
			     
				while(true){
					int len=in.read(buf, 0, buf.length);
					if(len<=0) break;
					out.write(buf, 0, len); //out.flush();
				}

				in.close();
				// send '\0'      
				buf[0]=0; out.write(buf, 0, 1); out.flush();
			} catch (IOException e) {
				// TODO Auto-generated catch block
//				e.printStackTrace();
			}    

		}
				
	}
	
	private void saveInWebServer(String[] args) {
		
		if(args.length == 0) return;
			
		String rdir, rfile;
		String lfile;
		
		String host, user;
		String password;
		
		// read WWW-server connection data from configuration file
		try {
			BufferedReader configFile = new BufferedReader(new InputStreamReader(new FileInputStream(new File(PaperToolkit.getToolkitRootPath(),
					"penSynch/WWW-config.txt"))));
			
			host = configFile.readLine();
			rdir = configFile.readLine() + "/";
			user = configFile.readLine();
			password = configFile.readLine();
			configFile.close();
			
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			return;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return;
		}
		
		//////////
		Session session;
		JSch jsch;
		try{
			jsch=new JSch();
			session=jsch.getSession(user, host, 22);
		}
		catch(Exception e){ 
	    	JOptionPane.showMessageDialog(null, "Failed to connect to Web Server.");
	    	return;
		}
		
		JPasswordField pwd = new JPasswordField(10);
		
		////////
		if(password != null){
			UserInfo ui=new LoginInfo(password);	      
			session.setUserInfo(ui);

			try {
				session.connect();
			} catch (JSchException e) {
				password = null;
			
				try {
					session=jsch.getSession(user, host, 22);
				} catch (JSchException e1) {
		    		JOptionPane.showMessageDialog(null, "Failed to connect to Web Server.");
		    		return;
				}
			}
		}
		////////
		
		while(password == null){    
		    int action = JOptionPane.showConfirmDialog(null, pwd,"Enter Password",JOptionPane.OK_CANCEL_OPTION);
		    if(action == JOptionPane.CANCEL_OPTION) return;
		    password = new String(pwd.getPassword());
		
		      // username and password will be given via UserInfo interface.
		    UserInfo ui=new LoginInfo(password);
		      
		    session.setUserInfo(ui);

		    try {
				session.connect();
			} catch (JSchException e) {
				password = null;
				
				try {
					session=jsch.getSession(user, host, 22);
				} catch (JSchException e1) {
			    	JOptionPane.showMessageDialog(null, "Failed to connect to Web Server.");
			    	return;
				}
			}
		}
		
		System.err.println("session connected....");
		
		for(int i=0; i<args.length; ++i){
			rfile = rdir + (new File(args[i]).getName());
			lfile = args[i];
			
			String command="scp -p -t "+rfile;
			
			try {
				executeCommand(lfile, rfile, command, session);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (JSchException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
				
		session.disconnect();
	}

	
	private boolean executeCommand(String lfile, String rfile, String command, Session session) throws IOException, JSchException{
		Channel channel=session.openChannel("exec");
		((ChannelExec)channel).setCommand(command);

		// get I/O streams for remote scp
		OutputStream out=channel.getOutputStream();    
		InputStream in=channel.getInputStream();
		channel.connect();
		      
		System.err.println("channel connected....");

		if(checkAck(in)!=0){	 
			JOptionPane.showMessageDialog(null, "Failed to connect to Web Server.");
			return false;
		}

		// send "C0644 filesize filename", where filename should not include '/'
		long filesize=(new File(lfile)).length();
		command="C0644 "+filesize+" ";
		if(lfile.lastIndexOf('/')>0){
			command+=lfile.substring(lfile.lastIndexOf('/')+1);
		}
		else{
			command+=lfile;
		}
		      
		command+="\n";
		    
		out.write(command.getBytes()); out.flush();
		if(checkAck(in)!=0){
			JOptionPane.showMessageDialog(null, "Failed to connect to Web Server.");
			return false;
		}

		// send a content of lfile    
		FileInputStream fis=new FileInputStream(lfile);      
		byte[] buf=new byte[1024];
		     
		while(true){
			int len=fis.read(buf, 0, buf.length);
			if(len<=0) break;
			out.write(buf, 0, len); //out.flush();
		}

		fis.close();
		fis=null;
		// send '\0'      
		buf[0]=0; out.write(buf, 0, 1); out.flush();
		      
		if(checkAck(in)!=0){
			return false;      
		}
		     
		out.close();
		channel.disconnect();
		
		return true;
	}
	
	
	class LoginInfo implements UserInfo{

		String password;
		public LoginInfo(String pass){
			password = pass;
		}
		
		public String getPassphrase() {
			// TODO Auto-generated method stub
			return "malakies";
		}

		public String getPassword() {
			return password;
		}

		public boolean promptPassphrase(String arg0) {
			// TODO Auto-generated method stub
			return true;
		}

		public boolean promptPassword(String arg0) {
			// TODO Auto-generated method stub
			return true;
		}

		public boolean promptYesNo(String arg0) {
			// TODO Auto-generated method stub
			return true;
		}

		public void showMessage(String arg0) {
			// TODO Auto-generated method stub
			
		}
    	  
      };
	
	static int checkAck(InputStream in) throws IOException{
	    int b=in.read();
	    // b may be 0 for success,
	    //          1 for error,
	    //          2 for fatal error,
	    //          -1
	    if(b==0) return b;
	    if(b==-1) return b;

	    if(b==1 || b==2){
	      StringBuffer sb=new StringBuffer();
	      int c;
	      do {
		c=in.read();
		sb.append((char)c);
	      }
	      while(c!='\n');
	      if(b==1){ // error
		System.out.print(sb.toString());
	      }
	      if(b==2){ // fatal error
		System.out.print(sb.toString());
	      }
	    }
	    return b;
	  }
	
	
	/**
	 * Send the data to the monitors that we have placed in the
	 * PaperToolkit/penSynch/RegisteredBatchedMonitors/ directory.
	 */
	private void sendToRegisteredMonitors(String[] args) {
		// find all files in PaperToolkit/penSynch/RegisteredBatchedMonitors/
		final File monitorsDir = new File(PaperToolkit.getToolkitRootPath(),
				"/penSynch/RegisteredBatchedMonitors/");
		final List<File> monitors = FileUtils.listVisibleFiles(monitorsDir);
		for (File f : monitors) {
			// DebugUtils.println(f.getAbsolutePath());
			String fileName = f.getName().toLowerCase();
			if (fileName.endsWith(".jar")) {
				// this works if your JRE install has associated JAR files with the double-click
				// action
				DebugUtils.println("Running JAR: " + fileName);
				SystemUtils.runJar(f, new String[] { '"' + new File(
						"penSynch/RegisteredBatchedMonitors/Examples/Example.txt").getAbsolutePath() + '"' });
			} else if (fileName.endsWith(".bat") || fileName.endsWith(".exe")) {
				DebugUtils.println("Running BAT/EXE: " + fileName);
				SystemUtils.run(f, new String[] { '"' + new File(
						"penSynch/RegisteredBatchedMonitors/Examples/Example.txt").getAbsolutePath() + '"' });
			} else if (fileName.endsWith(".monitor")) {
				DebugUtils.println("Running Monitor: " + f.getAbsolutePath());
				runMonitorFromConfigFile(f, args);
			} else {
				// unhandled file
			}
		}
	}

	/**
	 * @param args
	 */
	private void sendToBatchedDataServer(String[] args) {

		try {
			DebugUtils.println("Running the Batched Pen Data Importer");
			for (String arg : args) {
				DebugUtils.println("Argument: " + arg);
			}

			// find all registered listeners, and run them in sequence, with the correct arguments

			// open a socket connection to the batch importer / event handler
			// if an app is running, it will handle the incoming batched ink...
			// if not, this will simply throw an exception and fall through...
			final Socket socket = new Socket("localhost", BatchedDataDispatcher.DEFAULT_PLAINTEXT_PORT);
			final BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));

			// send over the xml file! =)
			// the path!
			bw.write("XML: " + args[0] + SystemUtils.LINE_SEPARATOR);
			bw.write(BatchedDataDispatcher.EXIT_COMMAND + SystemUtils.LINE_SEPARATOR);
			bw.flush();
			bw.close();

			// close the socket connection...
			socket.close();
		} catch (ConnectException e) {
			DebugUtils.println("No Batched Importer is Currently Listening to Port: "
					+ BatchedDataDispatcher.DEFAULT_PLAINTEXT_PORT);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			DebugUtils.println(e.getLocalizedMessage());
		} catch (UnknownHostException e) {
			e.printStackTrace();
			DebugUtils.println(e.getLocalizedMessage());
		} catch (IOException e) {
			e.printStackTrace();
			DebugUtils.println(e.getLocalizedMessage());
		}
	}

	/**
	 * Adds a monitor into the configuration file. This monitor will be run every time a digital pen is
	 * docked.
	 */
	public static void registerMonitor(BatchedDataImportMonitor monitor) {
		DebugUtils.println("Registering " + monitor.getName());

		// this is the name of the class
		DebugUtils.println(monitor.getClass());

		// this should be the run dir
		DebugUtils.println(monitor.getClass().getResource("/"));

		// create a new instance of this, and run the handler....
		// do it by name...
		try {

			Class<?> c = Class.forName("edu.stanford.hci.r3.demos.batched.monitor.BatchedMonitor");
			Object newInstance = c.newInstance();

		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}

		// figure out its path / classpath...

		// save it to our configuration file

	}

	/**
	 * The BatchImporter.exe calls this through a wrapper JAR...
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		new BatchedDataImporter(args);
	}

	/**
	 * Reads in the file, figures out which directory our monitor resides in, and runs it, with the correct
	 * classpath!
	 * 
	 * @param monitorConfigFile
	 *            the *.monitor configuration file, in Java Properties (non-xml) format.
	 * @param args
	 */
	private static void runMonitorFromConfigFile(File monitorConfigFile, String[] args) {
		Properties p = new Properties();
		try {
			p.load(new FileInputStream(monitorConfigFile));

			String mainClass = p.getProperty("MainClass");
			String eclipseProjectDirectory = p.getProperty("EclipseProjectDirectory");

			DebugUtils.println(mainClass);
			DebugUtils.println(eclipseProjectDirectory);

			// either a relative, or absolute path...
			File resolvedProjectDirectory = new File(eclipseProjectDirectory);
			if (resolvedProjectDirectory.exists()) {
				// assume it's absolute, do nothing
			} else {
				resolvedProjectDirectory = new File(PaperToolkit.getToolkitRootPath(),
						eclipseProjectDirectory);
			}

			EclipseProjectClassLoader eclipesProjectClassLoader = new EclipseProjectClassLoader(
					resolvedProjectDirectory);
			try {
				Class<?> classObj = eclipesProjectClassLoader.loadClass(mainClass);
				Method m = classObj.getMethod("main", new Class[] { args.getClass() });
				m.setAccessible(true);
				int mods = m.getModifiers();
				if (m.getReturnType() != void.class || !Modifier.isStatic(mods) || !Modifier.isPublic(mods)) {
					throw new NoSuchMethodException("main");
				}
				// call the main class
				m.invoke(null, new Object[] { args });

			} catch (ClassNotFoundException e) {
				System.err.println("BatchedDataImporter:: Could not find the class: " + mainClass);
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		DebugUtils.println(p.toString());
	}

	/**
	 * @param monitor
	 */
	public static void removeMonitor(BatchedDataImportMonitor monitor) {
		// TODO Auto-generated method stub
	}
}
